import os

from dotenv import load_dotenv
from sqlalchemy import MetaData
from sqlalchemy.ext.asyncio import AsyncSession, create_async_engine, async_sessionmaker
from sqlalchemy.orm import declarative_base

from app.models import models

load_dotenv()

# Async session maker
DATABASE_URL = os.getenv("DATABASE_URL")
engine = create_async_engine(DATABASE_URL, echo=True)
metadata = MetaData()
Base = declarative_base(metadata=metadata)
async_session = async_sessionmaker(autocommit=False, autoflush=False, bind=engine, class_=AsyncSession)


async def init_models():
    async with engine.begin() as conn:
        for model in [models]:
            await conn.run_sync(model.metadata.drop_all)
    async with engine.begin() as conn:
        for model in [models]:
            await conn.run_sync(model.metadata.create_all)


async def get_session() -> AsyncSession:
    async with async_session() as session:
        yield session


if __name__ == "__main__":
    import asyncio

    loop = asyncio.get_event_loop()

    try:
        loop.run_until_complete(init_models())
        print("Database initialized successfully")
    except Exception as e:
        print(f"Error initializing database: {e}")

    loop.close()
